@@ -92,9 +92,9 @@ for (FAST in c(FALSE, TRUE))
   }
 
   # Test right-only option
-  L <- irlba(A, 2, tol=1e-3, right_only=TRUE, fastpath=FAST)
+  L <- irlba(A, 2, tol=1e-3, right_only=TRUE, fastpath=FAST, work=20)
   S <- svd(A, nu=2, nv=2)
-  if (!isTRUE(all.equal(L$d, S$d[1:2])))
+  if (isTRUE(max(L$d - S$d[1:2]) > 1e-3))
   {
     stop("Failed right_only test", " fastpath=", FAST)
   }
@@ -177,6 +177,18 @@ for (FAST in c(FALSE, TRUE))
   {
     stop("Failed centering test (n > m) fastpath=", FAST)
   }
+
+  # test for https://github.com/bwlewis/irlba/issues/42
+  set.seed(1234)
+  a <- matrix(rnorm(10000), ncol=20)
+  center <- runif(ncol(a))
+  scale <- runif(ncol(a))
+  L <- irlba(a, 5, scale=scale, center=center, fastpath=FAST)
+  S <- svd(scale(a, center=center, scale=scale))
+  if (isTRUE(max(abs(S$d[1:5] - L$d)) > 1e-3))
+  {
+    stop("Failed scale + center test fastpath=", FAST)
+  }
 }
 
 # smallest=TRUE, m > n  (fastpath always FALSE in this case)
@@ -196,3 +208,15 @@ if (!isTRUE(all.equal(L$d, tail(svd(x)$d, 5))))
 {
   stop("Failed smallest svd test")
 }
+
+# test for https://github.com/bwlewis/irlba/issues/47 (again, fastpath always FALSE)
+set.seed(2345)
+a <- spMatrix(50, 40, x=runif(200), i=sample(50, 200, replace=TRUE), j=sample(40, 200, replace=TRUE))
+center <- runif(ncol(a))
+scale <- runif(ncol(a))
+L <- irlba(a, 5, scale=scale, center=center)
+S <- svd(scale(a, center=center, scale=scale))
+if (isTRUE(max(abs(S$d[1:5] - L$d)) > 1e-3))
+{
+  stop("Failed scale + center test for non-fastpath'able matrices")
+}
